﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Xml.Linq;

namespace GeocodingGogoCS
{
    public partial class Page : UserControl
    {
        public Page()
        {
            InitializeComponent();
        }

        //メッセージを取得、または設定する
        private string Message
        {
            set { this.textBlockMessage.Text = value; }
            get { return this.textBlockMessage.Text; }
        }

        //価格取得ボタン押下時のイベント・ハンドラ
        private void buttonGetPrice_Click(object sender, RoutedEventArgs e)
        {
            //価格取得処理を呼び出す
            GetPrice();
        }

        //テキストボックスで、キー押下時のイベント・ハンドラ
        private void textBoxAddress_KeyDown(object sender, KeyEventArgs e)
        {
            //Enterキー以外は処理しない
            if (e.Key != Key.Enter)
                return;

            //価格取得処理を呼び出す
            GetPrice();
        }

        //価格取得処理
        private void GetPrice()
        {
            //住所の入力チェック
            if (this.textBoxAddress.Text == string.Empty)
            {
                MessageBox.Show("住所が未入力です。");
                return;
            }

            //Geocodingを代理呼び出しするGoogle App EngineのURL設定
            //(住所をエンコードして渡す)
            string url = string.Format("http://wsproxy.appspot.com/?q={0}",
                HttpUtility.UrlEncode(this.textBoxAddress.Text));

            //GeocodingをWebClientで呼び出す
            WebClient geocodingClient = new WebClient();
            geocodingClient.DownloadStringCompleted
                += geocodingClient_DownloadStringCompleted;
            geocodingClient.DownloadStringAsync(new Uri(url));

            //メッセージの設定
            Message = "座標の取得中";
        }

        //座標取得完了時のイベント・ハンドラ
        void geocodingClient_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            //メッセージの設定
            Message = "座標の取得完了";

            //結果のXMLをパースする
            XElement rootElement = XElement.Parse(e.Result);

            //Webサービス上でのエラーチェック
            if (rootElement.Element("error") != null)
            {
                MessageBox.Show("座標の取得時にエラーが発生しました。");
                Message = string.Empty;
                return;
            }

            //住所が1つに特定できない場合のチェック
            if (rootElement.Element("choices") != null)
            {
                MessageBox.Show("座標の候補が複数あります。");
                Message = string.Empty;
                return;
            }

            //緯度経度を含む要素を取得する
            XElement coordinateElements = XElement.Parse(e.Result).Element("coordinate");

            //GoGoラボを呼び出すURLの設定
            //(緯度経度を渡す)
            string url = string.Format("http://api.gogo.gs/v1.2/?apid=fdotnet&lat={0}&lon={1}&sort=d",
                coordinateElements.Element("lat").Value,
                coordinateElements.Element("lng").Value);

            //GoGoラボをWebClientで呼び出す
            WebClient gogoClient = new WebClient();
            gogoClient.DownloadStringCompleted += gogoClient_DownloadStringCompleted;
            gogoClient.DownloadStringAsync(new Uri(url));

            //メッセージを設定する
            Message = "検索中";
        }

        //価格検索完了時のイベント・ハンドラ
        void gogoClient_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            //メッセージを設定する
            Message = "検索完了";

            //結果のXMLをパースする
            XElement rootElement = XElement.Parse(e.Result);

            //取得結果XMLをIEnumerable<Item>コレクションとして設定
            IEnumerable<Item> items = from i in rootElement.Elements("Item")
                                      select new Item
                                      {
                                          ShopCode = i.Element("ShopCode").Value,
                                          Brand = i.Element("Brand").Value,
                                          ShopName = i.Element("ShopName").Value,
                                          Latitude = i.Element("Latitude").Value,
                                          Longitude = i.Element("Longitude").Value,
                                          Distance = int.Parse(i.Element("Distance").Value),
                                          Address = i.Element("Address").Value,
                                          Price = int.Parse(i.Element("Price").Value),
                                          CheckDate = i.Element("Date").Value,
                                          Photo = i.Element("Photo").Value,
                                          Rtc = i.Element("Rtc").Value,
                                          Self = i.Element("Self").Value
                                      };

            //データグリッドにIEnumerable<Item>コレクションをバインド
            this.dataGridList.ItemsSource = items.ToList<Item>();

            //メッセージをクリア
            Message = string.Empty;

            //アイテムの取得結果がない場合メッセージを表示する
            if (items.Count() == 0)
                MessageBox.Show("検索結果がありませんでした。");
        }
    }
}
